/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : CIRCULAR SINGLY LINKED LIST
    *  Language: ANSI C
    *  File    : demo_CList-2.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


#include <stddef.h>
#include <stdio.h>
#include <assert.h>

#include <xc/CList.h>
#include <xc/xalloc.h>

#undef	CL_Node
#define	CL_Node	Node	/* mandatory when !GNU CC or strict ANSI */

typedef struct NodeRec *Node;
typedef struct ListRec *List;

struct NodeRec
{
    Node next;
    int  data;
};

struct ListRec
{
    Node link;
};

#define L1	"List: ("
#define L2	" )\n"

static void print(List list);
static void put(const char *s);

#if 1
#define TRACE__(item)	_MBEGIN puts("\
----------------------------------------------------------------------\r\
-------------------- " #item " "); item; _MEND
#else
#define TRACE__(item)	item
#endif

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT! */
extern void __xcint(wait)(void);		/* DON'T USE IT! */

int main()
{
    List   l;
    Node   n, mark;
    size_t len;

    COMM("CIRCULAR SINGLY LINKED LIST",
	 "LISTE CIRCULAIRE SIMPLEMENT CHAINEE");

    l = xnew(List);
    TRACE__(CL_INIT(l));
    print(l);

#ifdef __GLIBC__

    printf("FIRST: %p\n", CL_FIRST(l));

    {
	struct NodeRec const *last;

	CL_PLAST(l, last);

	printf("LAST:  %p\n", last);
    }

#endif

    (n = xnew(Node))->data = 1;
    TRACE__(CL_APPEND(l, n));
    print(l);

    mark = n;

    (n = xnew(Node))->data = 2;
    TRACE__(CL_APPEND(l, n));
    print(l);

    (n = xnew(Node))->data = 3;
    TRACE__(CL_INSERT(l, n));
    print(l);

    (n = xnew(Node))->data = 4;
    TRACE__(CL_INSERT_BEFORE(l, n, mark));
    print(l);

    (n = xnew(Node))->data = 5;
    TRACE__(CL_INSERT_AFTER(n, mark));
    print(l);

#if defined __GNUC__ && !defined __STRICT_ANSI__
    TRACE__(len = CL_LENGTH(l));
#else
    TRACE__(CL_LENGTH(l, len));
#endif
    printf("length = %lu\n", (unsigned long int)len);

    TRACE__(CL_PSELECT(l, 2, n));
    TRACE__(CL_REMOVE(l, n));
    xfree(n);
    print(l);

    TRACE__(CL_PSELECT(l, 0, n));
    TRACE__(CL_REMOVE(l, n));
    xfree(n);
    print(l);

    TRACE__(CL_CLEAN(l, xfree(__node); ));
    print(l);

    TRACE__(CL_ACTION(l, xfree(__node); ));
    xfree(l);

    WAIT();

    return 0;
}

static void print(List const list)
{
    struct NodeRec const *n;

    assert(list != NULL);

    put(L1);

    if (list->link != NULL)
    {
	printf(" %d", list->link->data);

#if 1
	CL_FOREACH_NODE(list, printf(" %d", __node->data); );
#else
	CL_FOREACH(list, n) printf(" %d", n->data);
#endif
    }

    put(L2);

    n = CL_FIRST(list);

    if (n != NULL) printf("First: %d\n", n->data);

    CL_PLAST(list, n);

    if (n != NULL) printf("Last:  %d\n", n->data);
}

static void put(const char *s)
{
    fputs(s, stdout);
}


/* End of file */
