/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	CLISTD:		CIRCULAR SINGLY LINKED LIST WITH DUMMY NODE  */


#ifndef __XC_CListD_H
#define __XC_CListD_H


/* Standard C Library: */
#include <stddef.h>
#include <assert.h>

/* eXtended C Library: */
#include <xc/version.h>		/* VERSION OF EXTENDED C LIBRARY	     */


/*****************************************************************************
 * CUSTOMIZATION PART
 *****************************************************************************/


#ifndef CLD_NodeRec
#define CLD_NodeRec CLD_NodeRec
typedef struct node CLD_NodeRec;
#endif

#ifndef CLD_ListRec
#define CLD_ListRec CLD_ListRec
typedef struct list CLD_ListRec;
#endif

#ifndef CLD_Node
#define CLD_Node     CLD_Node
typedef CLD_NodeRec *CLD_Node;
#endif

#ifndef CLD_List
#define CLD_List     CLD_List
typedef CLD_ListRec *CLD_List;
#endif


#ifndef CLD_next
#define CLD_next	next
#endif

#ifndef CLD_link
#define CLD_link	link
#endif


/*****************************************************************************
 * INTERNALS
 *****************************************************************************/


#if defined __GNUC__ && !defined __STRICT_ANSI__
#define CLD_TYPEREC(NODEREC)	typeof (NODEREC)
#define CLD_TYPE(NODE)		typeof (NODE)
#else
#define CLD_TYPEREC(NODEREC)	CLD_NodeRec
#define CLD_TYPE(NODE)		CLD_Node
#endif


#define	__CLD_NEXT(NODE)	((NODE)->CLD_next)
#define	__CLD_LINK(LIST)	((LIST)->CLD_link)


#define __CLD_FIRST(L)	__CLD_NEXT(L)


#define __CLD_SANITY_CHECK(L) assert					      \
									      \
 ( (L) != NULL && (							      \
									      \
 __CLD_NEXT(L) != NULL ) )


#define _CLD_INIT(L)							      \
_MBEGIN									      \
    assert((L) != NULL);						      \
									      \
    __CLD_NEXT(L) = (L);						      \
_MEND

#define _CLD_FOREACH_COND(L, NODE, COND)				      \
									      \
 for (									      \
									      \
 __CLD_SANITY_CHECK(L),							      \
									      \
 (NODE) = __CLD_FIRST(L); (NODE) != (L) && (COND); (NODE) = __CLD_NEXT(NODE))

#define _CLD_FOREACH(L, NODE)	_CLD_FOREACH_COND(L, NODE, 1)

#define _CLD_FOREACH_NODE(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) __node;					      \
									      \
    _CLD_FOREACH(L, __node) { INST }					      \
_MEND

#define _CLD_NEXT(L, N)	(__CLD_NEXT(N) != (L) ? __CLD_NEXT(N) : NULL)

#define	__CLD_PREV(L, NODE, PREV)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) ____n;					      \
									      \
    (PREV) = (L);							      \
									      \
    _CLD_FOREACH_COND(L, ____n, ____n != (NODE)) (PREV) = ____n;	      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define	_CLD_PREV(L, NODE)						      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(NODE) ___prev;				      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    __CLD_PREV(L, NODE, ___prev);					      \
									      \
    ___prev != (L) ? ___prev : NULL;					      \
_MEND
#else
#define	_CLD_PREV(L, NODE, PREV)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(NODE) ___prev;				      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    __CLD_PREV(L, NODE, ___prev);					      \
									      \
    (PREV) = ___prev != (L) ? ___prev : NULL;				      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define	_CLD_PPREV(L, NODE, PREV)	((PREV) = _CLD_PREV(L, NODE))
#else
#define	_CLD_PPREV(L, NODE, PREV)	_CLD_PREV(L, NODE, PREV)
#endif

#define _CLD_FIRST(L)							      \
									      \
 (__CLD_SANITY_CHECK(L), __CLD_FIRST(L) != (L) ? __CLD_FIRST(L) : NULL)

#define	__CLD_LAST(L, LAST)						      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) ____last;					      \
									      \
    __CLD_PREV(L, L, ____last);						      \
									      \
    (LAST) = ____last;							      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define	_CLD_LAST(L)							      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) ___last;					      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    __CLD_LAST(L, ___last);						      \
									      \
    ___last != (L) ? ___last : NULL;					      \
_MEND
#else
#define	_CLD_LAST(L, LAST)						      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) ___last;					      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    __CLD_LAST(L, ___last);						      \
									      \
    (LAST) = ___last != (L) ? ___last : NULL;				      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define	_CLD_PLAST(L, LAST)	((LAST) = _CLD_LAST(L))
#else
#define	_CLD_PLAST(L, LAST)	_CLD_LAST(L, LAST)
#endif

#define _CLD_APPEND(L, NODE)						      \
_MBEGIN									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    _CLD_INSERT_BEFORE(L, NODE, L);					      \
_MEND

#define _CLD_INSERT(L, NODE)						      \
_MBEGIN									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    _CLD_INSERT_AFTER(NODE, L);						      \
_MEND

#define _CLD_PREPEND(L, NODE) _CLD_INSERT(L, NODE)

#define _CLD_INSERT_BEFORE(L, NODE, MARK)				      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(NODE) ___prev_node;				      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLD_NEXT(NODE) = (MARK);						      \
									      \
    __CLD_PREV(L, MARK, ___prev_node);					      \
									      \
    __CLD_NEXT(___prev_node) = (NODE);					      \
_MEND

#define _CLD_INSERT_AFTER(NODE, MARK)					      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLD_NEXT(NODE) = __CLD_NEXT(MARK);				      \
									      \
    __CLD_NEXT(MARK) = (NODE);						      \
_MEND

#define __CLD_REMOVE(L, NODE)						      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(NODE) ___prev_node;				      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    assert((NODE) != NULL);						      \
									      \
    __CLD_PREV(L, NODE, ___prev_node);					      \
									      \
    __CLD_NEXT(___prev_node) = __CLD_NEXT(NODE);			      \
									      \
    __CLD_NEXT(NODE) = NULL;						      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLD_REMOVE(L, NODE)	(__CLD_REMOVE(L, NODE), (NODE))
#else
#define _CLD_REMOVE(L, NODE)	__CLD_REMOVE(L, NODE)
#endif

#define _CLD_EXTRACT(L, NODE) _CLD_REMOVE(L, NODE)

#define _CLD_FOREWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) __node, ___next_node;			      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    for (__node = __CLD_FIRST(L); __node != (L); __node = ___next_node)	      \
    {									      \
	___next_node = __CLD_NEXT(__node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _CLD_BACKWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) __node, ___prev_node;			      \
									      \
    __CLD_SANITY_CHECK(L);						      \
									      \
    __CLD_LAST(L, __node);						      \
									      \
    for (; __node != (L); __node = ___prev_node)			      \
    {									      \
	__CLD_PREV(L, __node, ___prev_node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _CLD_ACTION(L, INST)		_CLD_FOREWARD_ACTION(L, INST)
#define _CLD_FWD_ACTION(L, INST)	_CLD_FOREWARD_ACTION(L, INST)
#define _CLD_BWD_ACTION(L, INST)	_CLD_BACKWARD_ACTION(L, INST)

#define _CLD_CLEAN(L, INST)						      \
_MBEGIN									      \
    _CLD_ACTION(L, INST);						      \
									      \
    _CLD_INIT(L);							      \
_MEND

#define _CLD_PURGE(L, INST)	_CLD_PURGE(L, INST)

#define __CLD_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __i = 0;					      \
									      \
    _CLD_FOREACH_NODE(L, __i++;);					      \
									      \
    (LEN) = __i;							      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLD_LENGTH(L)							      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLD_LENGTH(L, __len);						      \
									      \
    __len;								      \
_MEND
#else
#define _CLD_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLD_LENGTH(L, __len);						      \
									      \
    (LEN) = __len;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLD_PLENGTH(L, LEN)	((LEN) = _CLD_LENGTH(L))
#else
#define _CLD_PLENGTH(L, LEN)	_CLD_LENGTH(L, LEN)
#endif

#define __CLD_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER size_t const __i = (INDEX);				      \
    __XC_REGISTER size_t       __j = 0;					      \
									      \
    _CLD_FOREACH(L, NODE) if (__j++ == __i) break;			      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLD_SELECT(L, INDEX)						      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) __node;					      \
									      \
    __CLD_SELECT(L, INDEX, __node);					      \
									      \
    __node;								      \
_MEND
#else
#define _CLD_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER CLD_TYPE(L) __node;					      \
									      \
    __CLD_SELECT(L, INDEX, __node);					      \
									      \
    (NODE) = __node;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLD_PSELECT(L, INDEX, NODE)	((NODE) = _CLD_SELECT(L, INDEX))
#else
#define _CLD_PSELECT(L, INDEX, NODE)	_CLD_SELECT(L, INDEX, NODE)
#endif


/*****************************************************************************
 * HIGH LEVEL MACROS
 *****************************************************************************/


#define _CLD_LINK(LIST)	__CLD_LINK((assert((LIST) != NULL), LIST))


#define CLD_INIT(LIST)				_CLD_INIT(_CLD_LINK(LIST))
#define CLD_NEXT(LIST, MARK)			_CLD_NEXT(_CLD_LINK(LIST), MARK)
#define CLD_FIRST(LIST)				_CLD_FIRST(_CLD_LINK(LIST))
#define CLD_APPEND(LIST, NODE)			_CLD_APPEND(_CLD_LINK(LIST), NODE)
#define CLD_INSERT(LIST, NODE)			_CLD_INSERT(_CLD_LINK(LIST), NODE)
#define CLD_PREPEND(LIST, NODE)			_CLD_PREPEND(_CLD_LINK(LIST), NODE)
#define CLD_INSERT_BEFORE(LIST, NODE, MARK)	_CLD_INSERT_BEFORE(_CLD_LINK(LIST), NODE, MARK)
#define CLD_INSERT_AFTER(NODE, MARK)		_CLD_INSERT_AFTER(NODE, MARK)
#define CLD_REMOVE(LIST, NODE)			_CLD_REMOVE(_CLD_LINK(LIST), NODE)
#define CLD_EXTRACT(LIST, NODE)			_CLD_EXTRACT(_CLD_LINK(LIST), NODE)
#define CLD_FOREACH_NODE(LIST, INST)		_CLD_FOREACH_NODE(_CLD_LINK(LIST), INST)
#define CLD_FOREACH(LIST, NODE)			_CLD_FOREACH(_CLD_LINK(LIST), NODE)
#define CLD_FOREACH_COND(LIST, NODE, COND)	_CLD_FOREACH_COND(_CLD_LINK(LIST), NODE, COND)
#define CLD_ACTION(LIST, INST)			_CLD_ACTION(_CLD_LINK(LIST), INST)
#define CLD_FWD_ACTION(LIST, INST)		_CLD_FWD_ACTION(_CLD_LINK(LIST), INST)
#define CLD_BWD_ACTION(LIST, INST)		_CLD_BWD_ACTION(_CLD_LINK(LIST), INST)
#define CLD_FOREWARD_ACTION(LIST, INST)		_CLD_FOREWARD_ACTION(_CLD_LINK(LIST), INST)
#define CLD_BACKWARD_ACTION(LIST, INST)		_CLD_BACKWARD_ACTION(_CLD_LINK(LIST), INST)
#define CLD_CLEAN(LIST, INST)			_CLD_CLEAN(_CLD_LINK(LIST), INST)
#define CLD_PURGE(LIST, INST)			_CLD_PURGE(_CLD_LINK(LIST), INST)

/*
 * Non portable versions
 */

#if defined __GNUC__ && !defined __STRICT_ANSI__

/* ====> Function behaviour */

#define CLD_PREV(LIST, MARK)			_CLD_PREV(_CLD_LINK(LIST), MARK)
#define CLD_LAST(LIST)				_CLD_LAST(_CLD_LINK(LIST))
#define CLD_LENGTH(LIST)			_CLD_LENGTH(_CLD_LINK(LIST))
#define CLD_SELECT(LIST, I)			_CLD_SELECT(_CLD_LINK(LIST), I)

#else

/* ====> strict ANSI conformance */

#define CLD_PREV(LIST, MARK, PREV)		_CLD_PREV(_CLD_LINK(LIST), MARK, PREV)
#define CLD_LAST(LIST, LAST)			_CLD_LAST(_CLD_LINK(LIST), LAST)
#define CLD_LENGTH(LIST, LEN)			_CLD_LENGTH(_CLD_LINK(LIST), LEN)
#define CLD_SELECT(LIST, I, NODE)		_CLD_SELECT(_CLD_LINK(LIST), I, NODE)

#endif

/*
 * Portable versions
 */

#define CLD_PPREV(LIST, MARK, PREV)		_CLD_PPREV(_CLD_LINK(LIST), MARK, PREV)
#define CLD_PLAST(LIST, LAST)			_CLD_PLAST(_CLD_LINK(LIST), LAST)
#define CLD_PLENGTH(LIST, LEN)			_CLD_PLENGTH(_CLD_LINK(LIST), LEN)
#define CLD_PSELECT(LIST, I, NODE)		_CLD_PSELECT(_CLD_LINK(LIST), I, NODE)


#endif /* XC CListD HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
