/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	LIST:		GENERIC LIST				     */


#ifndef __XC_LIST_H
#define __XC_LIST_H


/* Standard C Library: */
#include <stddef.h>
#include <stdio.h>

/* eXtended C Library: */
#include <xc/bool.h>		/* BOOLEAN TYPE				     */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


typedef const struct __xcint(List) *List;
typedef const struct __xcint(Node) *Node;

typedef Node Id;

typedef int (*ListActionProc)(Id id, void *data);
typedef int (*ListAction2Proc)(Id id1, Id id2, void *data);
typedef int (*ListSearchProc)(Id id, void *data);
typedef int (*ListSortProc)(Id id1, Id id2, void *data);
typedef int (*ListFileProc)(Id id, void *data, FILE *stream);

struct __xcint(LIST)
{
    List        (*const create)(size_t item_size);
    void        (*const clean)(List list);
    void        (*const destroy)(List list);
    Id          (*const append)(List list, const void *item);
    Id          (*const insert)(List list, const void *item);
    Id          (*const insert_after)(const void *item, Id mark);
    Id          (*const insert_before)(const void *item, Id mark);
    const void *(*const remove)(Id id);
    size_t      (*const length)(List list);
    bool        (*const is_empty)(List list);
    void        (*const action)(List list, ListActionProc proc, const void *data);
    void        (*const fwd_action)(List list, ListActionProc proc, const void *data);
    void        (*const bwd_action)(List list, ListActionProc proc, const void *data);
    void        (*const foreward_action)(List list, ListActionProc proc, const void *data);
    void        (*const backward_action)(List list, ListActionProc proc, const void *data);
    void        (*const action2)(List list, ListAction2Proc proc, const void *data);
    void        (*const action22)(List list1, List list2, ListAction2Proc proc, const void *data);
    Id          (*const search)(List list, ListSearchProc proc, const void *data);
    Id          (*const foreward_search)(List list, ListSearchProc proc, const void *data);
    Id          (*const backward_search)(List list, ListSearchProc proc, const void *data);
    void        (*const sort)(List list, ListSortProc proc, const void *data);
    Id          (*const first)(List list);
    Id          (*const last)(List list);
    Id          (*const next)(Node node);
    Id          (*const prev)(Node node);
    void       *(*const item)(Id id);
    List        (*const list)(Id id);
    Id          (*const node)(List list, const void *item);
    int         (*const write)(List list, ListFileProc proc, const void *data, FILE *stream);
    List        (*const read)(ListFileProc proc, const void *data, FILE *stream);
    void       *(*const alloc)(Id id, size_t item_size);
    size_t      (*const count)(List list);
    void       *(*const table)(List list, size_t item_size);
    Id          (*const move_to_begin)(Id id);
    Id          (*const move_to_end)(Id id);
    Id          (*const add)(List list, const void *item);
    const void *(*const del)(Id id);
    size_t      (*const len)(List list);
    void        (*const set_data)(List list, int entry, const void *data);
    void       *(*const get_data)(List list, int entry);
};

struct __xcint(List)
{
    const Node        head, tail;
};

struct __xcint(Node)
{
    void *const item;
    const List  list;
    const Node  prev, next;
};


extern const struct __xcint(LIST) __xclib(LIST);


#define LIST	__xclib(LIST)


#define def_rec_LIST(NAME, P, T)					      \
									      \
									      \
struct P##T##List;							      \
struct P##T##Node;							      \
									      \
typedef const struct P##T##List *T##List;				      \
typedef const struct P##T##Node *T##Node;				      \
									      \
typedef T##Node T##Id;							      \
									      \
typedef int (*T##ListActionProc)(T##Id id, void *data);			      \
typedef int (*T##ListAction2Proc)(T##Id id1, T##Id id2, void *data);	      \
typedef int (*T##ListSearchProc)(T##Id id, void *data);			      \
typedef int (*T##ListSortProc)(T##Id id1, T##Id id2, void *data);	      \
typedef int (*T##ListFileProc)(T##Id id, void *data, FILE *stream);	      \
									      \
struct __xcint(LIST__##NAME)						      \
{									      \
    T##List     (*const create)(size_t);				      \
    void        (*const clean)(T##List);				      \
    void        (*const destroy)(T##List);				      \
    T##Id       (*const append)(T##List, const T *);			      \
    T##Id       (*const insert)(T##List, const T *);			      \
    T##Id       (*const insert_after)(const T *, T##Id);		      \
    T##Id       (*const insert_before)(const T *, T##Id);		      \
    const T    *(*const remove)(T##Id);					      \
    size_t      (*const length)(T##List);				      \
    bool        (*const is_empty)(T##List);				      \
    void        (*const action)(T##List, T##ListActionProc, const void *);    \
    void        (*const fwd_action)(T##List, T##ListActionProc, const void *);\
    void        (*const bwd_action)(T##List, T##ListActionProc, const void *);\
    void        (*const foreward_action)(T##List, T##ListActionProc, const void *);\
    void        (*const backward_action)(T##List, T##ListActionProc, const void *);\
    void        (*const action2)(T##List, T##ListAction2Proc, const void *);  \
    void        (*const action22)(T##List, T##List, T##ListAction2Proc, const void *);\
    T##Id       (*const search)(T##List, T##ListSearchProc, const void *);    \
    T##Id       (*const foreward_search)(T##List, T##ListSearchProc, const void *);\
    T##Id       (*const backward_search)(T##List, T##ListSearchProc, const void *);\
    void        (*const sort)(T##List, T##ListSortProc, const void *);	      \
    T##Id       (*const first)(T##List);				      \
    T##Id       (*const last)(T##List);					      \
    T##Id       (*const next)(T##Id);					      \
    T##Id       (*const prev)(T##Id);					      \
    T          *(*const item)(T##Id);					      \
    T##List     (*const list)(T##Id);					      \
    T##Id       (*const node)(T##List, const void *);			      \
    int         (*const write)(T##List, T##ListFileProc, void *, FILE *);     \
    T##List     (*const read)(T##ListFileProc, const void *, FILE *);	      \
    T          *(*const alloc)(T##Id, size_t);				      \
    size_t      (*const count)(T##List);				      \
    T          *(*const table)(T##List, size_t);			      \
    T##Id       (*const move_to_begin)(T##Id);				      \
    T##Id       (*const move_to_end)(T##Id);				      \
    T##Id       (*const add)(T##List, const T *);			      \
    const T    *(*const del)(T##Id);					      \
    size_t      (*const len)(T##List);					      \
    void        (*const set_data)(T##List list, int entry, const void *data); \
    void       *(*const get_data)(T##List list, int entry);		      \
};									      \
									      \
struct P##T##List							      \
{									      \
    const T##Node  head, tail;						      \
};									      \
									      \
struct P##T##Node							      \
{									      \
    T      *const item;							      \
    const T##List list;							      \
    const T##Node prev, next;						      \
};									      \
									      \
									      \
static __unused const struct __xcint(LIST__##NAME) *NAME =		      \
									      \
	(const struct __xcint(LIST__##NAME) *)&__xclib(LIST)


#define def_LIST(NAME, P, T)						      \
									      \
									      \
struct P##T##List;							      \
struct P##T##Node;							      \
									      \
typedef const struct P##T##List *T##List;				      \
typedef const struct P##T##Node *T##Node;				      \
									      \
typedef T##Node T##Id;							      \
									      \
typedef int (*T##ListActionProc)(T##Id id, void *data);			      \
typedef int (*T##ListAction2Proc)(T##Id id1, T##Id id2, void *data);	      \
typedef int (*T##ListSearchProc)(T##Id id, void *data);			      \
typedef int (*T##ListSortProc)(T##Id id1, T##Id id2, void *data);	      \
typedef int (*T##ListFileProc)(T##Id id, void *data, FILE *stream);	      \
									      \
struct __xcint(LIST__##NAME)						      \
{									      \
    T##List     (*const create)(size_t);				      \
    void        (*const clean)(T##List);				      \
    void        (*const destroy)(T##List);				      \
    T##Id       (*const append)(T##List, T);				      \
    T##Id       (*const insert)(T##List, T);				      \
    T##Id       (*const insert_after)(T, T##Id);			      \
    T##Id       (*const insert_before)(T, T##Id);			      \
    T           (*const remove)(T##Id);					      \
    size_t      (*const length)(T##List);				      \
    bool        (*const is_empty)(T##List);				      \
    void        (*const action)(T##List, T##ListActionProc, const void *);    \
    void        (*const fwd_action)(T##List, T##ListActionProc, const void *);\
    void        (*const bwd_action)(T##List, T##ListActionProc, const void *);\
    void        (*const foreward_action)(T##List, T##ListActionProc, const void *);\
    void        (*const backward_action)(T##List, T##ListActionProc, const void *);\
    void        (*const action2)(T##List, T##ListAction2Proc, const void *);  \
    void        (*const action22)(T##List, T##List, T##ListAction2Proc, const void *);\
    T##Id       (*const search)(T##List, T##ListSearchProc, const void *);    \
    T##Id       (*const foreward_search)(T##List, T##ListSearchProc, const void *);\
    T##Id       (*const backward_search)(T##List, T##ListSearchProc, const void *);\
    void        (*const sort)(T##List, T##ListSortProc, const void *);	      \
    T##Id       (*const first)(T##List list);				      \
    T##Id       (*const last)(T##List list);				      \
    T##Id       (*const next)(T##Id);					      \
    T##Id       (*const prev)(T##Id);					      \
    T           (*const item)(T##Id);					      \
    T##List     (*const list)(T##Id);					      \
    T##Id       (*const node)(T##List, const void *item);		      \
    int         (*const write)(T##List, T##ListFileProc, void *, FILE *);     \
    T##List     (*const read)(T##ListFileProc, const void *, FILE *);	      \
    T           (*const alloc)(T##Id, size_t);				      \
    size_t      (*const count)(T##List);				      \
    T           (*const table)(T##List, size_t);			      \
    T##Id       (*const move_to_begin)(T##Id);				      \
    T##Id       (*const move_to_end)(T##Id);				      \
    T##Id       (*const add)(T##List, T);				      \
    T           (*const del)(T##Id);					      \
    size_t      (*const len)(T##List);					      \
    void        (*const set_data)(T##List list, int entry, const void *data); \
    void       *(*const get_data)(T##List list, int entry);		      \
};									      \
									      \
struct P##T##List							      \
{									      \
    const T##Node head, tail;						      \
};									      \
									      \
struct P##T##Node							      \
{									      \
    T             item;							      \
    const T##List list;							      \
    const T##Node prev, next;						      \
};									      \
									      \
									      \
static __unused const struct __xcint(LIST__##NAME) *NAME =		      \
									      \
	(const struct __xcint(LIST__##NAME) *)&__xclib(LIST)


#endif /* XC LIST HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
