/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : PRIMARY FUNCTIONS
  | Version : 0.3
  | Language: ANSI C
  | File    : base/puterrmsg.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

 [English]

 [French]

 Cette implmentation base sur alloca() permet d'assurer en partie l'atomicit
 de l'opration d'criture en effectuant un unique appel  une fonction de
 sortie de la bibliothque C: vfprintf().
 Par contre au niveau systme, rien ne garantit encore que la sortie se fera
 sans mlange avec un autre contexte d'excution. Pour pallier cette
 difficult, la version suivante utilisera en plus l'appel writev().

\*****************************************************************************/


#define _XCLIB_INTERNALS

#include "config.h"

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if !HAVE_ALLOCA_H
#    ifdef _AIX
#      pragma alloca
#    endif
#  endif
#endif

#include <sys/stat.h>

#include <limits.h>
#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if HAVE_ALLOCA_H
#    include <alloca.h>
#  else
#    ifndef _AIX
#      ifndef alloca
         extern char *alloca();
#      endif
#    endif
#  endif
#endif

#include <xc/mathdefs.h>
#include <xc/colors.h>
#include <xc/bool.h>
#include <xc/program.h>
#include <xc/missing.h>

#include "xc/base.h"

#include "global.h"

#define DEFAULT_OUTPUT	stderr

#define PREFIX1	"***"
#define PREFIX2	" **"

#define FG_COLOR_ENVIRONMENT	"ERROR_COLOR"
#define BG_COLOR_ENVIRONMENT	"ERROR_BG_COLOR"

#define FOREGROUND	1
#define BACKGROUND	0

#define STRINGSIZE(T)	(size_t)(sizeof (T) * CHAR_BIT * (M_LN2 / M_LN10) + 3)

#define __SET_STD_MSG()							      \
_MBEGIN									      \
    bool dot = false;							      \
									      \
    if (color)								      \
    {									      \
	set(foreground);						      \
									      \
	if (background != NULL)						      \
	{								      \
	    set(background);						      \
									      \
	    if (width > 0)						      \
	    {								      \
		register size_t i;					      \
									      \
		for (i = 0; i < width; i++) setc(' ');			      \
									      \
		setc('\r');						      \
	    }								      \
	}								      \
    }									      \
									      \
    if (program == NULL || program[0] != '\0')				      \
    {									      \
	if (program == NULL)						      \
	{								      \
	    set(null_string);						      \
	}								      \
	else								      \
	{								      \
	    set(program);						      \
	}								      \
									      \
	set(": ");							      \
    }									      \
									      \
    if (color) set(SGR_BOLD);						      \
									      \
    set(prefix);							      \
									      \
    setc(' ');								      \
									      \
    set(severity);							      \
									      \
    if (faci == NULL || faci[0] != '\0')				      \
    {									      \
	if (!dot)							      \
	{								      \
	    setc(':');							      \
									      \
	    dot = true;							      \
	}								      \
									      \
	setc(' ');							      \
									      \
	if (color) set(SGR_BOLD_OFF);					      \
	setc('[');							      \
	if (faci == NULL)						      \
	    set(null_string);						      \
	else								      \
	    set(faci);							      \
	setc(']');							      \
	if (color) set(SGR_BOLD);					      \
    }									      \
									      \
    if (text == NULL || text[0] != '\0')				      \
    {									      \
	if (!dot)							      \
	{								      \
	    setc(':');							      \
									      \
	    dot = true;							      \
	}								      \
									      \
	setc(' ');							      \
									      \
	if (text == NULL)						      \
	{								      \
	    set(null_string);						      \
	}								      \
	else								      \
	{								      \
	    set(text);							      \
	}								      \
    }									      \
									      \
    if (color) set(SGR_BOLD_OFF);					      \
									      \
    if (debug)								      \
    {	    								      \
	if (file == NULL || file[0] != '\0')				      \
	{								      \
	    set("\t- File ");						      \
									      \
	    if (file == NULL)						      \
		set(null_string);					      \
	    else							      \
	    {								      \
		setc('"');						      \
		if (color) set(SGR_BOLD);				      \
		set(file);						      \
		if (color) set(SGR_BOLD_OFF);				      \
		setc('"');						      \
	    }								      \
									      \
	    if (line > 0)						      \
	    {								      \
		char tmp[STRINGSIZE(int)];				      \
									      \
		if (snprintf(tmp, sizeof tmp, "%d", line) < 0) panic();	      \
									      \
		set(" at line ");					      \
		if (color) set(SGR_BOLD);				      \
		set(tmp);						      \
		if (color) set(SGR_BOLD_OFF);				      \
	    }								      \
	}								      \
    }									      \
									      \
    if (color) set(SGR_RESET);						      \
									      \
    setc('\n');								      \
_MEND

#define SET_STD_MSG()							      \
_MBEGIN									      \
    register const struct errmsg *errmsg;				      \
									      \
    if (output_is_tty && bell) setc('\a');				      \
									      \
    for (errmsg = list; errmsg != NULL; errmsg = errmsg->next)		      \
    {									      \
	const String faci = errmsg->faci;				      \
	const String file = errmsg->file;				      \
	const int    line = errmsg->line;				      \
	const String text = errmsg->text;				      \
									      \
	const String prefix = errmsg == list ? prefix1 : prefix2;	      \
									      \
	__SET_STD_MSG();						      \
    }									      \
_MEND

typedef const char *String;

typedef const String CString;

static String setup_color(String *bg_color_return);

static FILE *output = NULL;

void puterrmsg(CString severity, const struct errmsg *list, const va_list vargs)
{
    const char null_string[] = NULL_STRING;

    const char prefix1[] = PREFIX1;
    const char prefix2[] = PREFIX2;

    bool   output_is_ok;
    bool   output_is_tty;
    bool   color;
    bool   debug;
    bool   bell;
    String foreground, background;
    size_t width;
    String program;

    char *buffer;

    if (output == NULL) output = DEFAULT_OUTPUT;

    {
	struct stat status;

	output_is_ok = fstat(fileno(output), &status) != -1;

	output_is_tty = output_is_ok && S_ISCHR(status.st_mode);
    }

    color = _color == _always || (_color >= _auto && output_is_tty);

    if (color)
    {
	foreground = setup_color(&background);

	if (foreground == NULL)
	    color = false;
	else if (background != NULL)
	    width = 80;
    }

    debug = _debug == _always || (_debug >= _auto && getenv("DEBUG") != NULL);

    bell = getenv("ERROR_BELL") != NULL;

    program = debug ? program_invocation_name : program_invocation_short_name;

    {
	char ch[] = "?";

	size_t length = 0;

#define	set(s)	(length += strlen(s))
#define	setc(c)	(length += 1)
	SET_STD_MSG();
#undef	set
#undef	setc

	if ((buffer = alloca(length + 1)) == NULL) panic();

	buffer[0] = '\0';

#define	set(s)	strcat(buffer, s)
#define	setc(c)	(*ch = c, strcat(buffer, ch))
	SET_STD_MSG();
#undef	set
#undef	setc

	assert(strlen(buffer) == length);
    }

    if (fflush(output) == EOF) panic();

    if (fflush(NULL) == EOF) panic();

    if (vfprintf(output, buffer, vargs) < 0) panic();

    if (fflush(output) == EOF) panic();

#ifdef C_ALLOCA
    /* alloca(0); A N'UTILISER QUE DANS LE PROGRAMME!!! */
#endif
}

static String setup_color(String *bg_color_return)
{
    String foreground = NULL;
    String background = NULL;

    {
	String color_name;

	if ((color_name = getenv(FG_COLOR_ENVIRONMENT)) != NULL)
	    foreground = get_ansi_color_seq(color_name, FOREGROUND);

	if ((color_name = getenv(BG_COLOR_ENVIRONMENT)) != NULL)
	    background = get_ansi_color_seq(color_name, BACKGROUND);
    }

    if (bg_color_return != NULL) *bg_color_return = background;

    return foreground;
}


/* End Of File */
