/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : GENERIC LIST
  | Version : 0.3
  | Language: ANSI C
  | File    : list/list.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

[English]

[French]

  Le paramtre `item_size' de la mthode `create' sera uniquement utilis pour
  les opration d'criture et de lecture, s'il y a lieu.

\*****************************************************************************/


#define _XCLIB_INTERNALS

#define _FACILITY	FACILITY(list)

#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#define LL_Node	Node

#include <xc/List.h>
#include <xc/LList.h>
#include <xc/macros.h>
#include <xc/types.h>
#include <xc/error.h>
#include <xc/trace.h>
#include <xc/xalloc.h>

#include "xc/list.h"
#
#undef LIST

#include "global.h"

#define Action	ListActionProc
#define Action2	ListAction2Proc
#define Search	ListSearchProc
#define Sort	ListSortProc
#define File	ListFileProc

#define List	__List
#define Node	__Node

struct List;
struct Node;

typedef struct List *List;
typedef struct Node *Node;

static List        LIST_create(size_t item_size);
static void        LIST_clean(List list);
static void        LIST_destroy(List list);
static Node        LIST_append(List list, const void *item);
static Node        LIST_insert(List list, const void *item);
static Node        LIST_insert_after(const void *item, Node mark);
static Node        LIST_insert_before(const void *item, Node mark);
static const void *LIST_remove(Node node);
static size_t      LIST_length(List list);
static bool        LIST_is_empty(List list);
static void        LIST_action(List list, Action proc, const void *data);
static void        LIST_backward_action(List list, Action proc, const void *data);
static void        LIST_action2(List list, Action2 proc, const void *data);
static void        LIST_action22(List list1, List list2, Action2 proc, const void *data);
static Node        LIST_search(List list, Search proc, const void *data);
static Node        LIST_backward_search(List list, Search proc, const void *data);
static void        LIST_sort(List list, Sort proc, const void *data);
static Node        LIST_first(List list);
static Node        LIST_last(List list);
static Node        LIST_next(Node node);
static Node        LIST_prev(Node node);
static const void *LIST_item(Node node);
static List        LIST_list(Node node);
static Node        LIST_node(List list, const void *item);
static int         LIST_write(List list, File proc, const void *data, FILE *stream);
static List        LIST_read(File proc, const void *data, FILE *stream);
static void       *LIST_alloc(Node node, size_t size);
static size_t      LIST_count(List list);
static void       *LIST_table(List list, size_t item_size);
static Node        LIST_move_to_begin(Node node);
static Node        LIST_move_to_end(Node node);
static void        LIST_set_data(List list, int entry, const void *data);
static const void *LIST_get_data(List list, int entry);

#undef	List
#undef	Node

GLOBAL const struct __xcint(LIST) __xclib(LIST) =
{
    (List        (*)(size_t))				LIST_create,
    (void        (*)(List))				LIST_clean,
    (void        (*)(List))				LIST_destroy,
    (Node        (*)(List, const void *))		LIST_append,
    (Node        (*)(List, const void *))		LIST_insert,
    (Node        (*)(const void *, Node))		LIST_insert_after,
    (Node        (*)(const void *, Node))		LIST_insert_before,
    (const void *(*)(Node))				LIST_remove,
    (size_t      (*)(List))				LIST_length,
    (bool        (*)(List))				LIST_is_empty,
    (void        (*)(List, Action, const void *))	LIST_action,
    (void        (*)(List, Action, const void *))	LIST_action,
    (void        (*)(List, Action, const void *))	LIST_backward_action,
    (void        (*)(List, Action, const void *))	LIST_action,
    (void        (*)(List, Action, const void *))	LIST_backward_action,
    (void        (*)(List, Action2, const void *))	LIST_action2,
    (void        (*)(List, List, Action2, const void *))LIST_action22,
    (Node        (*)(List, Search, const void *))	LIST_search,
    (Node        (*)(List, Search, const void *))	LIST_search,
    (Node        (*)(List, Search, const void *))	LIST_backward_search,
    (void        (*)(List, Sort, const void *))		LIST_sort,
    (Node        (*)(List))				LIST_first,
    (Node        (*)(List))				LIST_last,
    (Node        (*)(Node))				LIST_next,
    (Node        (*)(Node))				LIST_prev,
    (void       *(*)(Node))				LIST_item,
    (List        (*)(Node))				LIST_list,
    (Node        (*)(List, const void *))		LIST_node,
    (int         (*)(List, File, const void *, FILE *))	LIST_write,
    (List        (*)(File, const void *, FILE *))	LIST_read,
    (void       *(*)(Node, size_t))			LIST_alloc,
    (size_t      (*)(List))				LIST_count,
    (void       *(*)(List, size_t))			LIST_table,
    (Node        (*)(Node))				LIST_move_to_begin,
    (Node        (*)(Node))				LIST_move_to_end,
    (Node        (*)(List, const void *))		LIST_append,
    (const void *(*)(Node))				LIST_remove,
    (size_t      (*)(List))				LIST_length,
    (void        (*)(List, int, const void *))		LIST_set_data,
    (void       *(*)(List, int))			LIST_get_data
};

struct Item
{
    size_t size;
};

typedef struct Item Item;

#undef	List
#undef	Node
#define List	__List
#define Node	__Node

#undef	L_Node
#define	L_Node	Data

struct DataRec
{
    struct DataRec *next;
    int             name;
    const void     *this;
};

typedef struct DataRec *Data;

struct List
{
    Node        head, tail;	/* EXPORTED */
    Item        item;
    Data        data;
};

struct Node
{
    const void *item;		/* EXPORTED */
    List        list;		/* EXPORTED */
    Node        prev, next;	/* EXPORTED */
};

#ifdef __TRACE__
# define RETURN	return trace_out,
#else
# define RETURN	return
#endif

static List LIST_create(size_t item_size)
{
    __trace_in__

    List list;

    list = xnew(List);

    LL_INIT(list);

    list->item.size = item_size;

    _L_INIT(list->data);

    RETURN list;

    __trace_out__
}

static void LIST_clean(List list)
{
    __trace_in__

    assert(list != NULL);

    LL_CLEAN(list, xfree(__node); );

    __trace_out__
}

static void LIST_destroy(List list)
{
    __trace_in__

    assert(list != NULL);

    LL_ACTION(list, xfree(__node); );

    if (list->data != NULL) xfree(list->data);

    xfree(list);

    __trace_out__
}

static Node LIST_append(List list, const void *item)
{
    __trace_in__

    Node node;

    assert(list != NULL);

    node = xnew(Node);

    node->list = list;
    node->item = item;

    LL_APPEND(list, node);

    RETURN node;

    __trace_out__
}

static Node LIST_insert(List list, const void *item)
{
    __trace_in__

    Node node;

    assert(list != NULL);

    node = xnew(Node);

    node->list = list;
    node->item = item;

    LL_INSERT(list, node);

    RETURN node;

    __trace_out__
}

static Node LIST_insert_after(const void *item, Node mark)
{
    __trace_in__

    List list;
    Node node;

    assert(mark != NULL);

    list = mark->list;

    assert(list != NULL);

    node = xnew(Node);

    node->list = list;
    node->item = item;

    LL_INSERT_AFTER(list, node, mark);

    RETURN node;

    __trace_out__
}

static Node LIST_insert_before(const void *item, Node mark)
{
    __trace_in__

    List list;
    Node node;

    assert(mark != NULL);

    list = mark->list;

    assert(list != NULL);

    node = xnew(Node);

    node->list = list;
    node->item = item;

    LL_INSERT_BEFORE(list, node, mark);

    RETURN node;

    __trace_out__
}

static const void *LIST_remove(Node node)
{
    __trace_in__

    List        list;
    const void *item;

    assert(node != NULL);

    list = node->list;

    assert(list != NULL);

    LL_REMOVE(list, node);

    item = node->item;

    xfree(node);

    RETURN item;

    __trace_out__
}

static size_t LIST_length(List list)
{
    __trace_in__

    size_t len;

    LL_PLENGTH(list, len);

    RETURN len;

    __trace_out__
}

static bool LIST_is_empty(List list)
{
    __trace_in__

    RETURN LL_FIRST(list) == NULL;

    __trace_out__
}

static void LIST_action(List list, Action proc, const void *data)
{
    __trace_in__

    assert(list != NULL && proc != NULL);

    LL_ACTION(list, if (proc((Id)__node, (void *)data)) break; );

    __trace_out__
}

static void LIST_backward_action(List list, Action proc, const void *data)
{
    __trace_in__

    assert(list != NULL && proc != NULL);

    LL_BACKWARD_ACTION(list, if (proc((Id)__node, (void *)data)) break; );

    __trace_out__
}

static void LIST_action2(List list, Action2 proc, const void *data)
{
    __trace_in__

    register Node node1, next1;
    register Node node2, next2;

    assert(list != NULL && proc != NULL);

    for (node1 = list->head; node1 != NULL; node1 = next1)
    {
	next1 = node1->next;

	for (node2 = next1; node2 != NULL; node2 = next2)
	{
	    next2 = node2->next;

	    if (proc((Id)node1, (Id)node2, (void *)data))
	    {
		next2 = NULL;
		next1 = NULL;
	    }
	}
    }

    __trace_out__
}

static void LIST_action22(List list1, List list2, Action2 proc, const void *data)
{
    __trace_in__

    register Node node1, next1;
    register Node node2, next2;

    assert(list1 != NULL && list2 != NULL && proc != NULL);

    for (node1 = list1->head; node1 != NULL; node1 = next1)
    {
	next1 = node1->next;

	for (node2 = list2->head; node2 != NULL; node2 = next2)
	{
	    next2 = node2->next;

	    if (proc((Id)node1, (Id)node2, (void *)data))
	    {
		next2 = NULL;
		next1 = NULL;
	    }
	}
    }

    __trace_out__
}

static Node LIST_search(List list, Search proc, const void *data)
{
    __trace_in__

    register Node node;

    assert(list != NULL && proc != NULL);

    for (node = list->head; node != NULL; node = node->next)
    {
	if (proc((Id)node, (void *)data)) break;
    }

    RETURN node;

    __trace_out__
}

static Node LIST_backward_search(List list, Search proc, const void *data)
{
    __trace_in__

    register Node node;

    assert(list != NULL && proc != NULL);

    for (node = list->tail; node != NULL; node = node->prev)
    {
	if (proc((Id)node, (void *)data)) break;
    }

    RETURN node;

    __trace_out__
}

#if !defined __GNUC__ || defined __STRICT_ANSI__

static Sort        sort_proc = NULL;
static const void *sort_data = NULL;

static int compare(void **element1, void **element2)
{
    return sort_proc(*element1, *element2, (void *)sort_data);
}

#endif

static void LIST_sort(List list, Sort proc, const void *data)
{
    __trace_in__

#if defined __GNUC__ && !defined __STRICT_ANSI__
    static int compare(void **element1, void **element2)
    {
	return proc(*element1, *element2, (void *)data);
    }
#endif

    Node *table;

    register Node   node;
    register size_t n, i;

    assert(list != NULL && proc != NULL);

#if defined __GNUC__ && !defined __STRICT_ANSI__
    n = LL_LENGTH(list);
#else
    LL_LENGTH(list, n);
#endif

    table = xmalloc(n * sizeof (Node));

    i = 0;

    for (node = list->head; node != NULL; node = node->next)
    {
	assert(i < n);

	table[i++] = node;
    }

    assert(i == n);

#if !defined __GNUC__ || defined __STRICT_ANSI__

    assert(sort_proc == NULL && sort_data == NULL);

    sort_proc = proc;
    sort_data = data;

#endif

    qsort(table, n, sizeof (Node), (compar_fn_t)compare);

#if !defined __GNUC__ || defined __STRICT_ANSI__
    sort_proc = NULL;
    sort_data = NULL;
#endif

    list->head = table[0];
    list->tail = table[n - 1];

    i = 0;

    for (node = list->head; node != NULL; node = node->next)
    {
	node->prev = i > 0     ? table[i - 1] : NULL;
	node->next = i < n - 1 ? table[i + 1] : NULL;

	++i;
    }

    xfree(table);

    __trace_out__
}

static Node LIST_first(List list)
{
    __trace_in__

    assert(list != NULL);

    RETURN LL_FIRST(list);

    __trace_out__
}

static Node LIST_last(List list)
{
    __trace_in__

    assert(list != NULL);

    RETURN LL_LAST(list);

    __trace_out__
}

static Node LIST_next(Node node)
{
    __trace_in__

    assert(node != NULL);

    RETURN LL_NEXT(node);

    __trace_out__
}

static Node LIST_prev(Node node)
{
    __trace_in__

    assert(node != NULL);

    RETURN LL_PREV(node);

    __trace_out__
}

static const void *LIST_item(Node node)
{
    __trace_in__

    assert(node != NULL);

    RETURN node->item;

    __trace_out__
}

static List LIST_list(Node node)
{
    __trace_in__

    assert(node != NULL);

    RETURN node->list;

    __trace_out__
}

static Node LIST_node(List list, const void *item)
{
    __trace_in__

    register Node node;

    assert(list != NULL);

    for (node = list->head; node != NULL; node = node->next)
    {
	if (node->item == item) break;
    }

    RETURN node;

    __trace_out__
}

static int LIST_write(List list, File proc, const void *data, FILE *stream)
{
    __trace_in__

    register Node node;

    assert(list != NULL && stream != NULL);

    if (proc == NULL && list->item.size == 0)
	RETURN EOF;

    if (proc == NULL)
    {
	if (fprintf(stream, "%lu\n", (unsigned long int)list->item.size) < 0)
	    RETURN EOF;
    }

    for (node = list->head; node != NULL; node = node->next)
    {
	if (proc != NULL)
	{
	    if (proc((Id)node, (void *)data, stream) == EOF)
		RETURN EOF;
	}
	else
	{
	    if (fwrite(node->item, list->item.size, 1, stream) != 1)
		RETURN EOF;
	}
    }

    RETURN 0;

    __trace_out__
}

static List LIST_read(File proc, const void *data, FILE *stream)
{
    __trace_in__

    List list;

    assert(stream != NULL);

    list = xnew(List);

    LL_INIT(list);

    list->item.size = 0;

    if (proc == NULL)
    {
	unsigned long int item_size;

	if (fscanf(stream, "%lu\n", &item_size) == EOF || item_size == 0)
	{
	    xfree(list);

	    RETURN NULL;
	}

	list->item.size = item_size;
    }

    for ever
    {
	register Node node;

	node = xnew(Node);

	node->list = list;

	if (proc != NULL)
	{
	    node->item = NULL;

	    if (proc((Id)node, (void *)data, stream) == EOF)
	    {
		xfree(node);

		break;
	    }
	}
	else
	{
	    void *item = xmalloc(list->item.size);

	    if (fread(item, list->item.size, 1, stream) < 1)
	    {
		xfree(item);

		break;
	    }

	    node->item = item;
	}

	LL_APPEND(list, node);
    }

    RETURN list;

    __trace_out__
}

static void *LIST_alloc(Node node, size_t item_size)
{
    __trace_in__

    assert(node != NULL && node->item == NULL && item_size > 0);

    node->item = item_size > 0 ? xmalloc(item_size) : NULL;

    RETURN (void *)node->item;

    __trace_out__
}

static size_t LIST_count(List list)
{
    __trace_in__

    size_t n;

#if defined __GNUC__ && !defined __STRICT_ANSI__
    n = LL_LENGTH(list);
#else
    LL_LENGTH(list, n);
#endif

    RETURN n;

    __trace_out__
}

static void *LIST_table(List list, size_t item_size)
{
    __trace_in__

    size_t count;

    assert(list != NULL);

#if defined __GNUC__ && !defined __STRICT_ANSI__
    count = LL_LENGTH(list);
#else
    LL_LENGTH(list, count);
#endif

    if (item_size > 0)
    {
	char *table;

	register char *dest;
	register Node  node;

	dest = table = xmalloc(count * item_size);

	for (node = list->head; node != NULL; node = node->next)
	{
	    memcpy(dest, node->item, item_size);

	    dest += item_size;
	}

	RETURN table;
    }
    else
    {
	const void **table;

	register size_t i;
	register Node   node;

	table = xmalloc(count * sizeof (const void *));

	i = 0;

	for (node = list->head; node != NULL; node = node->next)
	{
	    assert(i < count);

	    table[i++] = node->item;
	}

	assert(i == count);

	RETURN table;
    }

    __trace_out__
}

static Node LIST_move_to_begin(Node node)
{
    __trace_in__

    List list;

    assert(node != NULL);

    list = node->list;

    assert(list != NULL);

    LL_REMOVE(list, node);
    LL_INSERT(list, node);

    RETURN node;

    __trace_out__
}

static Node LIST_move_to_end(Node node)
{
    __trace_in__

    List list;

    assert(node != NULL);

    list = node->list;

    assert(list != NULL);

    LL_REMOVE(list, node);
    LL_APPEND(list, node);

    RETURN node;

    __trace_out__
}

static void LIST_set_data(List list, int name, const void *this)
{
    Data data;

    _L_FOREACH_COND(list->data, data, data->name != name);

    if (data == NULL)
    {
	if (this != NULL)
	{
	    data = xnew(Data);

	    data->name = name;
	    data->this = this;

	    _L_APPEND(list->data, data);
	}
    }
    else
    {
	if (this != NULL)
	{
	    data->this = this;
	}
	else
	{
	    _L_REMOVE(list->data, data);

	    xfree(data);
	}
    }
}

static const void *LIST_get_data(List list, int name)
{
    Data data;

    _L_FOREACH_COND(list->data, data, data->name != name);

    return data != NULL ? data->this : NULL;
}


/* End Of File */
