############################################################## -*- makefile -*-
#
#	Copyright (C) 1996-2001  Martin Vicente
#	This file is part of the eXtended C Library (XCLIB).
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Library General Public
#	License as published by the Free Software Foundation; either
#	version 2 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Library General Public License for more details.
#
#	You should have received a copy of the GNU Library General Public
#	License along with this library (see the file COPYING.LIB).
#	If not, write to the Free Software Foundation, Inc.,
#	59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################
#
#	CUSTOMIZATION OF CODE GENERATION
#
# Warning: This is an include file for GNU makefiles only.
#
#	Project : "Les outils de Martin"
#	Purpose : Options of code generation for GNU makefile
#	File    : utils/options.mk
#	Version : 1.1
#	Language: GNU Make
#	Author  : Martin Vicente
#	Contact : martin@vicente.org (or martin.vicente@free.fr)
#	Revision: Mon, 14 May 2001
#
# Description:
#
#   Defines code generation options from make parameters or environment.
#
# Environment:
#
#   TRACE   : Disable/Enable trace facility at compile time
#   DEBUG   : Disable/Enable generation of debugging information
#   OPTIMIZE: Disable/Enable code optimization
#
#	Valid values are: 0/No/False and 1/Yes/True
#
###############################################################################

ifneq (,)
This makefile requires GNU Make.
endif

ifndef   __UTILS_OPTIONS_MK
override __UTILS_OPTIONS_MK := OK

#==============================================================================

override UTILS ?= $(HOME)/devel/utils

include $(UTILS)/misc.mk
include $(UTILS)/utils.mk

#====( DEFINITIONS )

override OPT_SEE_HELP := "See $(MAKEFILE) or make help."

override OPT_DEVEL    = "Invalid \`DEVEL' (=\`$(DEVEL)') value. $(OPT_SEE_HELP)"
override OPT_TRACE    = "Invalid \`TRACE' (=\`$(TRACE)') value. $(OPT_SEE_HELP)"
override OPT_DEBUG    = "Invalid \`DEBUG' (=\`$(DEBUG)') value. $(OPT_SEE_HELP)"
override OPT_OPTIMIZE = "Invalid \`OPTIMIZE' (=\`$(OPTIMIZE)') value. $(OPT_SEE_HELP)"

ifdef DEVEL
  ifeq ($(DEVEL), 0)
    override DEVEL := 0
  else
  ifeq ($(DEVEL), 1)
    override DEVEL := 1
  else
    override DEVEL := $(shell echo $(DEVEL) | $(LOWER))
  ifeq ($(DEVEL), false)
    override DEVEL := 0
  else
  ifeq ($(DEVEL), true)
    override DEVEL := 1
  else
  ifeq ($(DEVEL), no)
    override DEVEL := 0
  else
  ifeq ($(DEVEL), yes)
    override DEVEL := 1
  else
  ifeq ($(DEVEL), off)
    override DEVEL := 0
  else
  ifeq ($(DEVEL), on)
    override DEVEL := 1
  else
    override BAD_SUFFIX = 1
    error::
      ifeq ($(SYSNAME), Linux)
	@ $(call Err, $(OPT_DEVEL))
      else
	@ $(ERR) $(bold)$(OPT_DEVEL)$(bold_off) $(END); exit 1
      endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
endif

ifdef TRACE
  ifeq ($(TRACE), 0)
    override TRACE := 0
  else
  ifeq ($(TRACE), 1)
    override TRACE := 1
  else
    override TRACE := $(shell echo $(TRACE) | $(LOWER))
  ifeq ($(TRACE), false)
    override TRACE := 0
  else
  ifeq ($(TRACE), true)
    override TRACE := 1
  else
  ifeq ($(TRACE), no)
    override TRACE := 0
  else
  ifeq ($(TRACE), yes)
    override TRACE := 1
  else
  ifeq ($(TRACE), off)
    override TRACE := 0
  else
  ifeq ($(TRACE), on)
    override TRACE := 1
  else
    override BAD_SUFFIX = 1
    error::
      ifeq ($(SYSNAME), Linux)
	@ $(call Err, $(OPT_TRACE))
      else
	@ $(ERR) $(bold)$(OPT_TRACE)$(bold_off) $(END); exit 1
      endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
endif

ifdef DEBUG
  ifeq ($(DEBUG), 0)
    override DEBUG := 0
  else
  ifeq ($(DEBUG), 1)
    override DEBUG := 1
  else
    override DEBUG := $(shell echo $(DEBUG) | $(LOWER))
  ifeq ($(DEBUG), false)
    override DEBUG := 0
  else
  ifeq ($(DEBUG), true)
    override DEBUG := 1
  else
  ifeq ($(DEBUG), no)
    override DEBUG := 0
  else
  ifeq ($(DEBUG), yes)
    override DEBUG := 1
  else
  ifeq ($(DEBUG), off)
    override DEBUG := 0
  else
  ifeq ($(DEBUG), on)
    override DEBUG := 1
  else
    override BAD_SUFFIX = 1
    error::
      ifeq ($(SYSNAME), Linux)
	@ $(call Err, $(OPT_DEBUG))
      else
	@ $(ERR) $(bold)$(OPT_DEBUG)$(bold_off) $(END); exit 1
      endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
endif

ifdef OPTIMIZE
  ifeq ($(OPTIMIZE), 0)
    override OPTIMIZE := 0
  else
  ifeq ($(OPTIMIZE), 1)
    override OPTIMIZE := 1
  else
    override OPTIMIZE := $(shell echo $(OPTIMIZE) | $(LOWER))
  ifeq ($(OPTIMIZE), false)
    override OPTIMIZE := 0
  else
  ifeq ($(OPTIMIZE), true)
    override OPTIMIZE := 1
  else
  ifeq ($(OPTIMIZE), no)
    override OPTIMIZE := 0
  else
  ifeq ($(OPTIMIZE), yes)
    override OPTIMIZE := 1
  else
  ifeq ($(OPTIMIZE), off)
    override OPTIMIZE := 0
  else
  ifeq ($(OPTIMIZE), on)
    override OPTIMIZE := 1
  else
    override BAD_SUFFIX = 1
    error::
      ifeq ($(SYSNAME), Linux)
	@ $(call Err, $(OPT_OPTIMIZE))
      else
	@ $(ERR) $(bold)$(OPT_OPTIMIZE)$(bold_off) $(END); exit 1
      endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
  endif
endif

override MAKEFILE ?= Makefile

    override OPTIONS    :=
    override T          :=
    override g          :=
    override O          :=

ifdef DEVEL
  ifeq ($(DEVEL), 1)
    override OPTIONS    += devel
    override DEPINCS    := $(DEPINCS_DEVEL)
    override DEPLIBS    := $(DEPLIBS_DEVEL)
    override DEFINES    := $(DEFINES_DEVEL)
    override INCLUDES   := $(INCLUDES_DEVEL)
    override LIBRARIES  := $(LIBRARIES_DEVEL)
    override DEFINES    += -D__DEVEL__
    override CC_OPTIONS += $(CC_DEVEL_OPTIONS)
  endif
  export DEVEL
endif

ifdef TRACE
  ifeq ($(TRACE), 1)
    override OPTIONS    += trace
    override DEFINES    += -D__TRACE__
    override T          := T
  endif
  export TRACE
endif

ifdef DEBUG
  ifeq ($(DEBUG), 1)
    override OPTIONS    += debug/2
    override DEFINES    += -D__DEBUG__
    override CC_OPTIONS += $(CC_DEBUG_OPTIONS)
    override g          := g2
  else
  ifeq ($(DEBUG), 0)
    override OPTIONS    += no-debug
    override DEFINES    += -DNDEBUG
    override LD_OPTIONS += -s
    override g          := g0
  endif
  endif
  export DEBUG
else
    override OPTIONS    += debug/1
    override g          := g1
endif

ifdef OPTIMIZE
  ifeq ($(OPTIMIZE), 1)
    override OPTIONS    += optimize
    override DEFINES    += -D__OPTIMIZE__
    override CC_OPTIONS += $(CC_OPTIMIZE_OPTIONS)
    override O          := O
  else
  ifeq ($(OPTIMIZE), 0)
    override OPTIONS    += no-optimize
    override CC_OPTIONS += -O0
  endif
  endif
  export OPTIMIZE
endif

override OPTIONS := $(strip $(OPTIONS))

override SUFFIX := __$T$g$O

#==============================================================================

endif  # __UTILS_OPTIONS_MK

# [END OF FILE]
