############################################################## -*- makefile -*-
#
#	Copyright (C) 1996-2001  Martin Vicente
#	This file is part of the eXtended C Library (XCLIB).
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Library General Public
#	License as published by the Free Software Foundation; either
#	version 2 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Library General Public License for more details.
#
#	You should have received a copy of the GNU Library General Public
#	License along with this library (see the file COPYING.LIB).
#	If not, write to the Free Software Foundation, Inc.,
#	59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
###############################################################################
#
#	IMPLICIT RULES
#
# Warning: This is an include file for GNU makefiles only.
#
#	Project : "Les outils de Martin"
#	Purpose : Implicit rules for GNU makefile
#	File    : utils/rules-suff.mk
#	Version : 1.1
#	Language: GNU Make
#	Author  : Martin Vicente
#	Contact : martin@vicente.org (or martin.vicente@free.fr)
#	Revision: Mon, 14 May 2001
#
###############################################################################

ifneq (,)
This makefile requires GNU Make.
endif

ifndef   __UTILS_RULES_MK
override __UTILS_RULES_MK := OK

#==============================================================================

override UTILS ?= $(HOME)/devel/utils

include $(UTILS)/misc.mk
include $(UTILS)/mklog.mk

#====( DEFINITIONS )

ifndef BAD_SUFFIX

override S1 := $(strip $(S1))
override S2 := $(strip $(S2))

override MAKEDEP_SED_REPLACE =\
$(subst /,\/,$@ $(STATIC)/$(S1)$*$(S2)$(SUFFIX).o $(SHARED)/$(S1)$*$(S2)$(SUFFIX).o)

$(DEPDIR)/$(S1)%$(S2)$(SUFFIX).d: %.c
	@ if [ ! -d $(dir $@) ]; then $(MKDIR) $(dir $@); fi
    ifdef VERBOSE
      ifeq ($(SYSNAME), Linux)
	@ $(call Log, -- $(call Bold,)$(~makedep) $(<F))
      else
	@ $(LOG) -- $(bold)$(bold_off)$(~makedep) $(<F) $(END)
      endif
    endif
	- $(strip $(MAKEDEP.c) $< | sed 's/$*\.o/$(MAKEDEP_SED_REPLACE)/' > $@)
	@ [ -s $@ ] || $(RM) $@

$(STATIC)/$(S1)%$(S2)$(SUFFIX).o: %.c
	@ if [ ! -d $(dir $@) ]; then $(MKDIR) $(dir $@); fi
    ifdef VERBOSE
      ifeq ($(SYSNAME), Linux)
	@ $(call Log, -- $(call Bold, $(~compile) $(<F)))
      else
	@ $(LOG) -- $(bold)$(~compile) $(<F)$(bold_off) $(END)
      endif
    endif
	$(strip $(COMPILE.c) -o $@ $<)

$(SHARED)/$(S1)%$(S2)$(SUFFIX).o: override COMPILE.c += $(CC_PIC_OPTIONS)
$(SHARED)/$(S1)%$(S2)$(SUFFIX).o: %.c
	@ if [ ! -d $(dir $@) ]; then $(MKDIR) $(dir $@); fi
    ifdef VERBOSE
      ifeq ($(SYSNAME), Linux)
	@ $(call Log, -- $(call Bold, $(~compile_pic) $(<F)))
      else
	@ $(LOG) -- $(bold)$(~compile_pic) $(<F)$(bold_off) $(END)
      endif
    endif
	$(strip $(COMPILE.c) -o $@ $<)

%: $(STATIC)/$(S1)%$(S2)$(SUFFIX).o
    ifdef VERBOSE
      ifeq ($(SYSNAME), Linux)
	@ $(call Log, -- $(call Bold, $(~link) $@))
      else
	@ $(LOG) -- $(bold)$(~link) $@$(bold_off) $(END)
      endif
    endif
	$(strip $(LINK.o) -o $@ $^ $(LDLIBS))

ifdef VERBOSE
ifeq ($(SYSNAME), Linux)
override define BUILD_SHLIB_LOG
@ $(call Log, "-- "$(call Bold, $(~sharlib_pic) $@))
endef
else
override define BUILD_SHLIB_LOG
@ $(LOG) "-- "$(bold)$(~sharlib_pic) $@$(bold_off) $(END)
endef
endif
else
override define BUILD_SHLIB_LOG
endef
endif

override define BUILD_SHLIB_CMD
$(strip $(LINK.o) -o $@ $^ $(LDLIBS))
chmod -x $@
endef

override define BUILD_SHLIB
$(BUILD_SHLIB_LOG)
$(BUILD_SHLIB_CMD)
endef

(%$(SUFFIX).o): $(STATIC)/%$(SUFFIX).o
    ifdef VERBOSE
      ifeq ($(SYSNAME), Linux)
	@ $(call Log, -- $(~archive) $(<F))
      else
	@ $(LOG) -- $(~archive) $(<F) $(END)
      endif
    endif
	$(strip $(AR) $(ARFLAGS) $@ $<)

endif

#==============================================================================

endif  # __UTILS_RULES_MK

# [END OF FILE]
